/*
 * @(#)TileChangeListener.java	1.2 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

// @(#)TileChangeListener.java	1.2 97/10/14 10/14/97

package java.awt.image;

/**
  * An interface for objects that wish to be informed when tiles
  * of a WritableRenderedImage become modifiable by some writer via
  * a call to getWritableTile, and when they become unmodifiable via
  * the last call to releaseWritableTile.
  *
  * @see WritableRenderedImage
  * @see TileChangeFilter
  * @see TileChangeMulticaster
  *
  * @author Thomas DeWeese
  * @author Daniel Rice
  */
public interface TileChangeListener {

  /**
    * A tile is about to be grabbed for writing.
    *
    * @param source the image that owns the tile.
    * @param tileX the X index of the tile that has become modifiable.
    * @param tileY the Y index of the tile that has become modifiable.
    */
  public void tileGrabbed(WritableRenderedImage source,	
			  int tileX, int tileY);

  /**
    * A tile is no longer grabbed for writing by any object.
    *
    * @param source the image that owns the tile.
    * @param tileX the X index of the tile that has become modifiable.
    * @param tileY the Y index of the tile that has become modifiable.
    */
  public void tileReleased(WritableRenderedImage source,
			   int tileX, int tileY);
}
