/*
 * @(#)BasicButtonListener.java	1.22 98/02/23
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
 
package java.awt.swing.plaf.basic;

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.awt.swing.*;
import java.awt.swing.event.*;

/**
 * Button Listener
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate 
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.22 02/23/98
 * @author Jeff Dinkins 
 * @author Arnaud Weber (keyboard UI support)
 */

public class BasicButtonListener implements MouseListener, MouseMotionListener, 
                                   FocusListener, ChangeListener, 
                                   Serializable
{
    // used in mouseDragged
    private Rectangle tmpRect = new Rectangle();

    public BasicButtonListener(AbstractButton b) {
    }

    public void setupKeyboard(AbstractButton b) {
	ButtonModel bm = b.getModel();

	b.resetKeyboardActions();

	b.registerKeyboardAction(new PressedAction((AbstractButton)b), 
                                 KeyStroke.getKeyStroke(' ',0,false),
				 JComponent.WHEN_FOCUSED);

	b.registerKeyboardAction(new ReleasedAction((AbstractButton)b),
                                 KeyStroke.getKeyStroke(' ',0,true),
				 JComponent.WHEN_FOCUSED);


	if(bm.getMnemonic() != 0) {
	    b.registerKeyboardAction(new PressedAction((AbstractButton)b),
				     KeyStroke.getKeyStroke(bm.getMnemonic(),
                                                             ActionEvent.ALT_MASK,false),
				     JComponent.WHEN_IN_FOCUSED_WINDOW);
	    b.registerKeyboardAction(new ReleasedAction((AbstractButton)b),
				     KeyStroke.getKeyStroke(bm.getMnemonic(),
                                                             ActionEvent.ALT_MASK,true),
				     JComponent.WHEN_IN_FOCUSED_WINDOW);
            /** If the user release the alt mofifier while the key is down, we still need
             *  to release the button
             */
            b.registerKeyboardAction(new ReleasedAction((AbstractButton)b),
                                     KeyStroke.getKeyStroke(bm.getMnemonic(),
                                                             0,true),
                                     JComponent.WHEN_IN_FOCUSED_WINDOW);
	}
    }

    public void stateChanged(ChangeEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();
        b.repaint();

	// PENDING(jeff) - simple, but fairly poor, workaround for bug 4113876. 
	// This will be fixed properly for the next release.
	setupKeyboard(b);
    }

    public void focusGained(FocusEvent e) { 
	AbstractButton b = (AbstractButton) e.getSource();
        if (b instanceof JButton) {
            JRootPane root = b.getRootPane();
            if (root != null) {
                root.setDefaultButton((JButton)b);
            }
        }
	b.repaint();
    }

    public void focusLost(FocusEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();
	b.repaint();
    }

    public void mouseMoved(MouseEvent e) {
    };


    public void mouseDragged(MouseEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();

	// HACK! We're forced to do this since mouseEnter and mouseExit aren't
	// reported while the mouse is down.
	ButtonModel model = b.getModel();

	if(model.isPressed()) {
            tmpRect.width = b.getWidth();
            tmpRect.height = b.getHeight();
            if(tmpRect.contains(e.getPoint())) {
                model.setArmed(true);
            } else {
                model.setArmed(false);
            }
        }
    };

    public void mouseClicked(MouseEvent e) {
    };
 
    public void mousePressed(MouseEvent e) {
        if ( SwingUtilities.isLeftMouseButton(e) ) {
	    AbstractButton b = (AbstractButton) e.getSource();
	    ButtonModel model = b.getModel();

	    // But because of the mouseDragged hack above, we can't do setArmed
	    // in mouseEnter. As a workaround, set it here just before setting
	    // focus.
	    model.setArmed(true);
	    model.setPressed(true);
	    if(!b.hasFocus()) {
		b.requestFocus();
	    }            
	}
    };
    
    public void mouseReleased(MouseEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();
	ButtonModel model = b.getModel();
	model.setPressed(false);
    };
 
    public void mouseEntered(MouseEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();
	if(b.isRolloverEnabled()) {
	    b.getModel().setRollover(true);
	}
    };
 
    public void mouseExited(MouseEvent e) {
	AbstractButton b = (AbstractButton) e.getSource();
	if(b.isRolloverEnabled()) {
	    b.getModel().setRollover(false);
	}
    };
	
    static class PressedAction extends AbstractAction {
        AbstractButton owner;

        PressedAction(AbstractButton owner) {
	    super("pressedAction");
	    this.owner = owner;
	}
	
	public void actionPerformed(ActionEvent e) {
	    ButtonModel model = owner.getModel();
	    model.setArmed(true);
	    model.setPressed(true);
	    if(!owner.hasFocus()) {
		owner.requestFocus();
	    }
        }

	public boolean isEnabled() {
	    if(!owner.getModel().isEnabled()) {
		return false;
	    } else {
		return true;
	    }
	}
    }

   static class ReleasedAction extends AbstractAction {
        AbstractButton owner;

        ReleasedAction(AbstractButton owner) {
	    super("releasedAction");
	    this.owner = owner;
	}
	
	public void actionPerformed(ActionEvent e) {
	    ButtonModel model = owner.getModel();
	    model.setPressed(false);
        }

	public boolean isEnabled() {
	    if(!owner.getModel().isEnabled()) {
		return false;
	    } else {
		return true;
	    }
	}
    }

}

