/*
 * @(#)BasicFileChooserUI.java	1.30 98/02/03
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.basic;

import java.awt.swing.*;
import java.awt.swing.preview.*;
import java.awt.swing.preview.JDirectoryPane;
import java.awt.swing.event.*;
import java.awt.swing.plaf.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import java.util.*;

/**
 * A Basic L&F implementation of FileChooserUI.  
 *
 * @version 1.30 02/03/98
 * @author Ray Ryan
 * @author Jeff Dinkins (rewrote gui layout)
 */
public class BasicFileChooserUI extends FileChooserUI {
    protected BasicDirectoryPaneUI directoryPaneUI;

    protected JPanel centerPanel;

    protected JFileChooser fileChooser = null;
    protected JDirectoryPane directoryPane = null;
    protected Component accessory = null;
    
    protected Listener listener;

    protected Action goHomeAction = new GoHomeAction();
    protected Action mkdirAction = new MkdirAction();
    protected Action cancelAction = new CancelAction();
    protected Action okayAction = new OkayAction();

    protected TypesComboBoxModel typesComboBoxModel;

    private static Dimension PREF_SIZE = new Dimension(500, 300);
    private static Dimension MIN_SIZE = new Dimension(300, 200);

    // The following are private. BasicFileChooser is not meant to
    // be subtyped to become another L&F.
    private JComboBox currentDir;	

    private JButton okayButton;
    private JButton cancelButton;

    private JTextField filename;

    private JComboBox types;

    private JPanel bodyPanel = null;

    private Icon newFolderIcon = null;
    private Icon upFolderIcon = null;
    private Icon homeFolderIcon = null;
    private Icon listViewIcon = null;
    private Icon detailsViewIcon = null;

    private static final Dimension hstrut10 = new Dimension(10, 1);
    private static final Dimension hstrut25 = new Dimension(25, 1);
    private static final Dimension vstrut10 = new Dimension(1, 10);
    private static final Insets shrinkwrap = new Insets(0,0,0,0);

    //
    // ComponentUI Interface Implementation methods
    //
    public static ComponentUI createUI(JComponent c) {
        return new BasicFileChooserUI((JFileChooser)c);
    }

    public BasicFileChooserUI(JFileChooser b) {
	if(newFolderIcon == null) {
	    newFolderIcon = UIManager.getIcon("FileChooser.newFolder");
	    upFolderIcon = UIManager.getIcon("FileChooser.upFolder");
	    homeFolderIcon = UIManager.getIcon("FileChooser.homeFolder");
	    detailsViewIcon = UIManager.getIcon("FileChooser.detailsView");
	    listViewIcon = UIManager.getIcon("FileChooser.listView");
	}
    }
            
    /**
     * If necessary, create this UI's various components and its
     * BasicDirectoryPaneUI. Tell the sub UI to installUI (it take's
     * the "Center" spot on c's contentPane), then put buttons and
     * fields in the appropriate places around it. Establish listeners.
     * PENDING(jeff) rewrite so that this hasa instead of isa DirectoryPane
     */
    public void installUI(JComponent c) {
	fileChooser = (JFileChooser) c;
	directoryPane = fileChooser.getDirectoryPane();
	accessory = fileChooser.getAccessory();

	fileChooser.setLayout(new BoxLayout(fileChooser, BoxLayout.Y_AXIS));
	fileChooser.add(Box.createRigidArea(vstrut10));
	
	// ********************************* //
	// **** Construct the top panel **** //
	// ********************************* //

	// Folder manipulation buttons
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));

	// Add the top panel to the fileChooser
	fileChooser.add(topPanel);
	fileChooser.add(Box.createRigidArea(vstrut10));
	
	// ComboBox Label
	JLabel l = new JLabel("Look in:");
	l.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	l.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	topPanel.add(Box.createRigidArea(hstrut10));
	topPanel.add(l);
	topPanel.add(Box.createRigidArea(hstrut25));
	
	// CurrentDir ComboBox
	currentDir = new JComboBox();
	currentDir.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	currentDir.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	topPanel.add(currentDir);
	topPanel.add(Box.createRigidArea(hstrut10));
	
	// Up Button
	JButton b = new NoFocusButton(upFolderIcon);
	b.setToolTipText("Up One Level");
	b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	b.setMargin(shrinkwrap);
	b.setFocusPainted(false);
	b.addActionListener(directoryPane.getGoUpAction());
	topPanel.add(b);
	topPanel.add(Box.createRigidArea(hstrut10));
	
	// Home Button
	b = new NoFocusButton(homeFolderIcon);
	b.setToolTipText("Home");
	b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	b.setMargin(shrinkwrap);
	b.setFocusPainted(false);
	b.addActionListener(goHomeAction);
	topPanel.add(b);
	topPanel.add(Box.createRigidArea(hstrut10));
	
	// New Folder Button
	b = new NoFocusButton(newFolderIcon);
	b.setToolTipText("Create New Folder");
	b.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	b.setMargin(shrinkwrap);
	b.setFocusPainted(false);
	b.addActionListener(mkdirAction);
	topPanel.add(b);
	topPanel.add(Box.createRigidArea(hstrut10));
	
	// List Button
	JToggleButton tb = new JToggleButton(listViewIcon);
	tb.setToolTipText("List");
	tb.setEnabled(false);
	tb.setFocusPainted(false);
	tb.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	tb.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	tb.setMargin(shrinkwrap);
	topPanel.add(tb);
	
	// Details Button
	tb = new JToggleButton(detailsViewIcon);
	tb.setToolTipText("Details");
	tb.setFocusPainted(false);
	tb.setSelected(true);
	tb.setEnabled(false);
	tb.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	tb.setAlignmentY(JComponent.CENTER_ALIGNMENT);
	tb.setMargin(shrinkwrap);
	topPanel.add(tb);
	topPanel.add(Box.createRigidArea(hstrut10));

	// ************************************** //
	// **** Construct the directory pane **** //
	// ************************************** //
	centerPanel = new JPanel(new BorderLayout());
	centerPanel.add(directoryPane, BorderLayout.CENTER);
	if(accessory != null) {
	    centerPanel.add(accessory, BorderLayout.EAST);
	}
	fileChooser.add(centerPanel);

	// ********************************** //
	// **** Construct the bottom panel ** //
	// ********************************** //
	JPanel bottomPanel = new JPanel();
	bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.X_AXIS));
	bottomPanel.add(Box.createRigidArea(hstrut10));

	// Add the bottom panel to file chooser
	fileChooser.add(Box.createRigidArea(vstrut10));
	fileChooser.add(bottomPanel);
	fileChooser.add(Box.createRigidArea(vstrut10));
	
	// labels
	JPanel labelPanel = new JPanel();
	labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));

	l = new JLabel("File name:");
	labelPanel.add(l);

	labelPanel.add(Box.createRigidArea(vstrut10));

	l = new JLabel("Files of type:");
	labelPanel.add(l);

	bottomPanel.add(labelPanel);
	bottomPanel.add(Box.createRigidArea(hstrut25));

	// file entry and filters
	JPanel fileAndFilterPanel = new JPanel();
	fileAndFilterPanel.setLayout(new BoxLayout(fileAndFilterPanel, BoxLayout.Y_AXIS));

	filename = new JTextField();
	filename.addActionListener(okayAction);
	fileAndFilterPanel.add(filename);
	
	fileAndFilterPanel.add(Box.createRigidArea(vstrut10));

	typesComboBoxModel = createTypesComboBoxModel();
	types = new JComboBox(typesComboBoxModel);
	types.setRenderer(createTypeRenderer());
	fileAndFilterPanel.add(types);

	bottomPanel.add(fileAndFilterPanel);
	bottomPanel.add(Box.createRigidArea(hstrut10));

	// buttons
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
	
	okayButton = new JButton("Open") {
	    // PENDING(jeff) - this is a hack to get the open and cancel buttons to
	    // be the same width - this will break when the open and cancel buttons are
	    // internationalized. FIX THIS.
	    public Dimension getMaximumSize() {
		return new Dimension(75, super.getMaximumSize().height);
	    }
	};
	okayButton.addActionListener(okayAction);
	okayButton.setToolTipText("Open");
	buttonPanel.add(okayButton);
	buttonPanel.add(Box.createRigidArea(vstrut10));

	cancelButton = new JButton("Cancel") {
	    // PENDING(jeff) - this is a hack to get the open and cancel buttons to
	    // be the same width - this will break when the open and cancel buttons
	    // are internationalized. FIX THIS.
	    public Dimension getMaximumSize() {
		return new Dimension(75, super.getMaximumSize().height);
	    }
	};
	cancelButton.setToolTipText("Cancel");
	cancelButton.addActionListener(cancelAction);
	buttonPanel.add(cancelButton);

	bottomPanel.add(buttonPanel);
	bottomPanel.add(Box.createRigidArea(hstrut10));
	
	// **** Create the fileChooser property change listener **** //
	listener = createListener();
        directoryPane.addPropertyChangeListener(listener);
	directoryPane.getListSelectionModel().addListSelectionListener(listener);

    }   

    public void uninstallUI(JComponent c) {
	if(c != fileChooser) {
	    throw new IllegalComponentStateException(this + " was asked to deinstall() " 
		+ c + " when it only knows about " + fileChooser + "."); 
	}

        directoryPane.getModel().removePropertyChangeListener(listener);
	directoryPane.getListSelectionModel().removeListSelectionListener(listener);
	listener = null;

	centerPanel = null;
	directoryPane = null;
	fileChooser = null;
	accessory = null;

	fileChooser.removeAll();
	fileChooser = null;
    }
    
    public Dimension getPreferredSize(JComponent x)    {
	return PREF_SIZE;
    }
    
    public Dimension getMinimumSize(JComponent x)  {
	return MIN_SIZE;
    }
    
    public Dimension getMaximumSize(JComponent x) {
	return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }
   
    public void update() {
	DirectoryModel model = directoryPane.getModel();
	File curDir = model.getCurrentDirectory();

	currentDir.removeAllItems();
	currentDir.addItem(curDir.getName());
	mkdirAction.setEnabled(curDir.canWrite());

	// PENDING(jeff) I think this is wrong. Don't reset the types
	// every time an event is fired.
	typesComboBoxModel.setTypes(fileChooser.getChoosableFileTypes());
    }

    /**
     * A subclass of button that does not accept focus
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class NoFocusButton extends JButton {
	public NoFocusButton(Icon i) { super(i); }
	public boolean isFocusTraversable() {
	    return false;
	}
    }

    //
    // Renderer for Types ComboBox
    //
    protected TypeRenderer createTypeRenderer() {
	return new TypeRenderer();
    }

    /**
     * Render different type sizes and styles.
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    public class TypeRenderer extends BasicListCellRenderer {
	public Component getListCellRendererComponent(JList list,
	    Object value, int index, boolean isSelected, 
	    boolean cellHasFocus) {

	    super.getListCellRendererComponent(list, value,
		index, isSelected, cellHasFocus);

	    // PENDING(jeff) the typesComboBoxModel instance serves as
	    // a value in the ComboBox, to mark "no filter", b/c
	    // nulls make the ComboBox choke.
	    if (value == typesComboBoxModel) {
		setText("All Types (*.*)");
	    } else {
		setText(((FileType)value).getPresentationName());
	    }
	    return this;
	}
    }

    //
    // DataModel for Types Comboxbox
    //
    protected TypesComboBoxModel createTypesComboBoxModel() {
	return new TypesComboBoxModel();
    }

    /**
     * Data model for a type-face selection combo-box. 
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class TypesComboBoxModel extends AbstractListModel
    implements ComboBoxModel {
	boolean itIsI_PleaseDoNotPanic = false;
	Object value;
	Vector types = null;

	protected TypesComboBoxModel() {
	    super();
	    // PENDING(jeff) - Hack. "this" is a marker for "no filter",
	    // as JComboBox chokes on null
	    value = this;
	}

	protected void setTypes(Vector v) {
	    Vector oldTypes = types;
	    types = null;
	    if (oldTypes != null && !itIsI_PleaseDoNotPanic) {
		fireIntervalRemoved(this, 0, oldTypes.size()-1);
	    }
	    types = v;
	    if (v!= null && v.size() > 0 && !itIsI_PleaseDoNotPanic) {
		fireIntervalAdded(this, 0, v.size()-1);
	    }
	}

	//
	// ComboBoxModel
	//
	public void setSelectedItem(Object anObject) {
	    itIsI_PleaseDoNotPanic = true;
	    if (anObject == this) {
		directoryPane.getModel().setShownType(null);
	    } else {
		directoryPane.getModel().setShownType((FileType)anObject);
	    }
	    itIsI_PleaseDoNotPanic = false;
	}
	public Object getSelectedItem() {
	    Object t = directoryPane.getModel().getShownType();
	    return t == null ? this : t;
	}
	
	public int getSize() {
	    Vector v = types;
	    return v == null ? 1 : v.size()+1; // leave room for "*.*"
	}

	public Object getElementAt(int index) {
	    Vector v = types;
	    if (v == null) {
		return this;
	    } 

	    int count = v.size();
	    if (index >= count) {
		return this; 
	    } 

	    return v.elementAt(index);
	}
    }
	
    //
    // Listener class.
    //
    protected Listener createListener() {
	return new Listener();
    }

    /**
     * Acts on property changes.
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class Listener implements PropertyChangeListener, ListSelectionListener {
        public void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getPropertyName().equals("accessory")) {
		if(centerPanel != null) {
		    centerPanel.add((Component)evt.getNewValue(), BorderLayout.CENTER);
		}
	    } 
	    update();
	}

	public void valueChanged(ListSelectionEvent e) {
	    TypedFile f = directoryPane.getSelectedFile();
	    if (!e.getValueIsAdjusting() &&
		f != null && !f.getType().isContainer()) {
		filename.setText(f.getName());
	    }
	}
    }

    //
    // Action Classes
    //
    /**
     * Acts on the "home" key event or equivalent event.
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class GoHomeAction extends AbstractAction {
	protected GoHomeAction() {
	    super("Go Home");
	}
	public void actionPerformed(ActionEvent e) {
	    directoryPane.getModel().setCurrentDirectory(null);
	}
    }

    protected File mkdir(String name) {
	String path = directoryPane.getModel().getCurrentDirectory().getPath();
	File f = new File(path, name);
	int i = 1;

	while (!(!f.exists() && f.mkdirs()) && i < 100) {
	    f = new File(path, name + " " + i);
	    i++;
	}

	return f;
    }

    /**
     * Creates a new folder.
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class MkdirAction extends AbstractAction {
	protected MkdirAction() {
	    super("New Folder");
	}
	public void actionPerformed(ActionEvent e) {
	    String name = JOptionPane. showInputDialog(fileChooser, "Create a new folder named:");

	    if (name != null) {
		File newDir = mkdir(name);

		if (newDir.exists()) {
		    directoryPane.getModel().setCurrentDirectory(newDir);
		} else {
		    JOptionPane.showMessageDialog(fileChooser,
			"Could not create folder: \n" + name,
			"New Folder Failed", JOptionPane.ERROR_MESSAGE);
		}
	    }
	}
    }

    /**
     * Responds to a cancel request.
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class CancelAction extends AbstractAction {
	public void actionPerformed(ActionEvent e) {
	    fileChooser.performCancel();
	}
    }

    /**
     * Responds to "OK".
     * <p>
     * Warning: serialized objects of this class will not be compatible with
     * future swing releases.  The current serialization support is appropriate
     * for short term storage or RMI between Swing1.0 applications.  It will
     * not be possible to load serialized Swing1.0 objects with future releases
     * of Swing.  The JDK1.2 release of Swing will be the compatibility
     * baseline for the serialized form of Swing objects.
     */
    protected class OkayAction extends AbstractAction {
	public void actionPerformed(ActionEvent e) {
	    fileChooser.performOkay();
	}
    }
}
