/*
 * @(#)BasicTextUI.java	1.6 98/02/02
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package java.awt.swing.plaf.basic;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;

import java.awt.swing.*;
import java.awt.swing.text.*;
import java.awt.swing.plaf.*;

/**
 * Provides the look and feel features that are common across
 * the Basic text LAF implementations.  
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @author  Timothy Prinzing
 * @version 1.6 02/02/98
 */
public abstract class BasicTextUI extends DefaultTextUI {

    /**
     * Loads the given keymap with default settings
     * appropriate for the UI being created.  This
     * is called if a default keymap is being created
     * because an existing one wasn't found.  Subclasses
     * that reimplement this would generally want to 
     * call the superclass implementation to make sure
     * all of the appropriate bindings get loaded.
     *
     * This is implemented to load the set of keybindings
     * appropriate for all text components in motif.
     *
     * @param map the keymap
     */
    protected void loadDefaultKeymap(Keymap map) {
	JTextComponent.loadKeymap(map, defaultBindings, 
					 getComponent().getActions());
    }

    /**
     * Fetches the name of the keymap that
     * will be installed/used by default for this UI.
     * This is implemented to generate a name based upon
     * the getPropertyPrefix method, and will generally
     * need not be redefined.
     *
     * @return the name
     */
    public String getKeymapName() {
	return "Basic" + getPropertyPrefix();
    }

    /**
     * Creates the keymap to use for the text component, and installs
     * any necessary bindings into it.  By default, the keymap is
     * shared between all instances of the associated text component.
     * It has the name defined by the getKeymapName method.
     *
     * @return the keymap
     * @see #getKeymapName
     * @see java.awt.swing.text.JTextComponent
     */
    protected Keymap createKeymap() {
	String nm = getKeymapName();
	Keymap map = JTextComponent.getKeymap(nm);
	if (map == null) {
	    Keymap parent = JTextComponent.getKeymap(JTextComponent.DEFAULT_KEYMAP);
	    map = JTextComponent.addKeymap(nm, parent);
	    loadDefaultKeymap(map);
	}
	return map;
    }

    /**
     * Creates the view for an element that has been
     * broken as a result of formatting.
     *
     * @param elem the element
     * @param p0 the starting offset
     * @param p1 the ending offset
     * @return the view
     */
    public View create(Element elem, int p0, int p1) {
	throw new Error("unsupported");
    }

    /**
     * Default bindings all keymaps implementing the Basic feel.
     */
    static final JTextComponent.KeyBinding[] defaultBindings = {
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_C, 
							       InputEvent.CTRL_MASK),
				       DefaultEditorKit.copyAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_V, 
							       InputEvent.CTRL_MASK),
				       DefaultEditorKit.pasteAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_X, 
							       InputEvent.CTRL_MASK),
				       DefaultEditorKit.cutAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 
							       InputEvent.SHIFT_MASK),
				       DefaultEditorKit.selectionBackwardAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 
							       InputEvent.SHIFT_MASK),
				       DefaultEditorKit.selectionForwardAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 
							       InputEvent.CTRL_MASK),
				       DefaultEditorKit.beginAction),
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 
							       InputEvent.CTRL_MASK),
				       DefaultEditorKit.endAction)
    };


}
