/*
 * @(#)MetalComboBoxButton.java	1.10 98/02/06
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.metal;

import java.awt.*;
import java.awt.event.*;
import java.awt.swing.plaf.basic.*;
import java.awt.swing.*;
import java.awt.swing.plaf.*;
import java.awt.swing.border.*;
import java.io.Serializable;

/**
 * JButton subclass to help out MetalComboBoxUI
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @see MetalComboBoxButton
 * @version 1.10 02/06/98
 * @author Tom Santos
 */
public class MetalComboBoxButton extends JButton {
    protected JComboBox comboBox;
    protected Icon comboIcon;
    protected boolean iconOnly = false;
    protected boolean forceDrawFocus = false;

    public final JComboBox getComboBox() { return comboBox; }
    public final void setComboBox( JComboBox cb ) { comboBox = cb; }

    public final Icon getComboIcon() { return comboIcon; }
    public final void setComboIcon( Icon i ) { comboIcon = i; }

    MetalComboBoxButton() {
        super( "" );
    }

    public MetalComboBoxButton( JComboBox cb, Icon i ) {
	super( "" );

        comboBox = cb;
	comboIcon = i;
    }

    public MetalComboBoxButton( JComboBox cb, Icon i, boolean onlyIcon ) {
        this( cb, i );
        iconOnly = onlyIcon;
    }

    public Insets getInsets() {
        return comboBox != null ? comboBox.getInsets() : new Insets( 0, 0, 0, 0 );
    }

    public void paint( Graphics g ) {
        // Paint the button as usual
        super.paint( g );

	Dimension size = getSize();
	Insets insets = getInsets();
	size.width -= insets.left + insets.right;
	size.height -= insets.top + insets.bottom;
	int iconLeft = insets.left + size.width;

	// Paint the icon
	if ( comboIcon != null ) {
	    int iconWidth = comboIcon.getIconWidth();
	    final int RIGHT_BUFFER = 8;

	    if ( iconOnly ) {
	        iconLeft = (insets.left + (size.width / 2)) - (iconWidth / 2);
	    }
	    else {
	        iconLeft -= iconWidth + RIGHT_BUFFER;
	    }

	    int iconHeight = comboIcon.getIconHeight();
	    int iconTop = (insets.top + (size.height / 2)) - (iconHeight / 2);

	    comboIcon.paintIcon( this, g, iconLeft, iconTop );
	}

	// Let the renderer paint
	if ( ! iconOnly && comboBox != null) {
	    MetalComboBoxUI ui = (MetalComboBoxUI)comboBox.getUI();
	    ListCellRenderer renderer = comboBox.getRenderer();
	    Component c;
	    ui.validateMenu();
	    boolean renderPressed = getModel().isPressed();
	    c = renderer.getListCellRendererComponent(ui.getList(),
						      comboBox.getSelectedItem(), -1, renderPressed, false);
	    c.setFont(ui.getCurrentValuePane().getFont());

            if (model.isArmed() && model.isPressed()) {
	        if (isOpaque()) {
		    c.setBackground(UIManager.getColor("Button.pressed"));
		}
	    } else {
		c.setBackground(comboBox.getBackground());
	    }
	    c.setForeground(comboBox.getForeground());

	    ui.getCurrentValuePane().paintComponent( g, c, this, insets.left+4, insets.top+4,
						     (iconLeft - 8) - insets.left,
						     insets.top + size.height - 8 );
	}
    }

    public boolean hasFocus() {
        return comboBox.isEditable() ? super.hasFocus() : forceDrawFocus;
    }

    public void forceDrawFocus( boolean b ) {
        forceDrawFocus = b;
    }

    public boolean isFocusTraversable() {
        return comboBox.isEditable() ? true : false;
    }

    public void requestFocus() {
/*
        if ( comboBox.isEditable() ) {
	    super.requestFocus();
	}
*/
    }

    public void updateUI() {
        setUI( new MetalComboBoxButtonUI( this ) );
    }

    class MetalComboBoxButtonUI extends MetalButtonUI {
        MetalComboBoxButtonListener newListener;
        MetalComboBoxButtonKeyListener keyListener;
        MetalComboBoxButtonFocusListener focusListener;

        public MetalComboBoxButtonUI( MetalComboBoxButton button ) {
	    newListener = new MetalComboBoxButtonListener( button );
	    keyListener = new MetalComboBoxButtonKeyListener( button );
	    focusListener = new MetalComboBoxButtonFocusListener( button );
	}

        public void installUI( JComponent c ) {
	    super.installUI( c );

	    removeMouseListener( listener );
	    removeMouseMotionListener( listener );
	    addMouseListener( newListener );
	    addMouseMotionListener( (MouseMotionListener)newListener );
	    addKeyListener( keyListener );
	    addFocusListener( focusListener );
	}

        public void uninstallUI( JComponent c ) {
	    removeMouseListener( newListener );
	    removeMouseMotionListener( (MouseMotionListener)newListener );
	    removeKeyListener( keyListener );
	    removeFocusListener( focusListener );
	    super.uninstallUI( c );
	}
    }

    class MetalComboBoxButtonKeyListener extends KeyAdapter 
            implements Serializable {
        MetalComboBoxButton button;

        public MetalComboBoxButtonKeyListener( MetalComboBoxButton b ) {
	    button = b;
	}

        public void keyReleased( KeyEvent e ) {
	    if ( e.getKeyCode() == KeyEvent.VK_SPACE ) {
	        MetalComboBoxUI ui = (MetalComboBoxUI)button.getComboBox().getUI();
		if ( ui.popupIsVisible() ) {
		    ui.hidePopup();
		}
		else {
		    ui.showPopup();
		}
	    }
	}
    }


    class MetalComboBoxButtonFocusListener extends FocusAdapter implements Serializable {
        MetalComboBoxButton button;

        public MetalComboBoxButtonFocusListener( MetalComboBoxButton b ) {
	    button = b;
	}

        public void focusLost( FocusEvent e ) {
/*
	    button.getModel().setPressed( false );
	    button.validate();
	    button.invalidate();
	    button.paintImmediately( 0, 0, button.getSize().width, button.getSize().height );	
*/    
	}
    }

    class MetalComboBoxButtonListener extends BasicButtonListener {
        MetalComboBoxButton button;
        boolean isDraggingProtection = false;

        public MetalComboBoxButtonListener( MetalComboBoxButton aButton ) {
	    super( aButton );
	    button = aButton;
	}

        public void mousePressed(MouseEvent anEvent) {
	    super.mousePressed( anEvent );

	    MetalComboBoxUI ui = (MetalComboBoxUI)button.getComboBox().getUI();

	    if ( isDraggingProtection ) {
	        return;
	    }
	    else {
	        isDraggingProtection = true;
	    }

	    if(!comboBox.isEnabled())
	        return;

	    if(!SwingUtilities.isLeftMouseButton(anEvent))
	        return;

	    if ( ui.popupIsVisible() ) {
	        ui.hidePopup();
	    }
	    else {
	        ui.showPopup();
	    }
	}
      
        public void mouseReleased(MouseEvent anEvent) {
	    isDraggingProtection = false;
	    MetalComboBoxUI ui = (MetalComboBoxUI)button.getComboBox().getUI();
	    if ( !(SwingUtilities.getLocalBounds(button).contains(anEvent.getPoint())) ) {
	        ui.doMouseReleased( anEvent );
	    }
	    super.mouseReleased( anEvent );
	}

        public void mouseDragged(MouseEvent anEvent) {
	    MetalComboBoxUI ui = (MetalComboBoxUI)button.getComboBox().getUI();
	    if ( !(SwingUtilities.getLocalBounds(button).contains(anEvent.getPoint())) ) {
	        ui.doMouseDragged( anEvent );
	    }
	}
    };
}

