/*
 * @(#)MetalProgressBarUI.java	1.7 98/02/05
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.metal;

import java.awt.swing.plaf.basic.*;
import java.awt.swing.*;
import java.awt.swing.event.*;
import java.awt.*;
import java.awt.swing.plaf.*;

/**
 * The Metal implementation of ProgressBarUI.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.7 02/05/98
 * @author Michael C. Albers
 */
public class MetalProgressBarUI extends BasicProgressBarUI {
    // used in paint()
    Dimension size, innerSize;
    int x, y, length;
    int min, max;
    JProgressBar progressBar;
    BoundedRangeModel model;
    Insets b;
    long span;
    double bigWidth, bigHeight, bigValue, fractionComplete;

    // override Basic behavior - needs "get" methods.
    static protected final Dimension PREFERRED_INNER_HORIZONTAL 
                                     = new Dimension(144, 12);
    static protected final Dimension PREFERRED_INNER_VERTICAL
                                     = new Dimension(12, 144);

 
    public static ComponentUI createUI(JComponent c) {
	return new MetalProgressBarUI();
    }

    /**
     * The default size of a horizontal progress bar
     */
    public Dimension getPreferredInnerHorizontal() {
	return PREFERRED_INNER_HORIZONTAL;
    }

    /**
     * The default size of a vertical progress bar
     */
    public Dimension getPreferredInnerVertical() {
	return PREFERRED_INNER_VERTICAL;
    }

    /**
     * Don't leave space between the interior and the border
     */
    public int getBorderBuffer() {
	return 0;
    }
    
    /**
     * The default length of a cell
     */
    public int getCellLength() {
	return 1;
    }
    
    /**
     * Gap between cells, or the lack thereof
     */
    public int getCellSpacing() {
	return 0;
    }


    // NOTE (mca): If the default mechanism was appropriately applied to 
    // BasicProgressBarUI, I probably wouldn't any code here at all.
    public void paint(Graphics g, JComponent c) {
	size = c.getSize();
	progressBar = (JProgressBar)c;
	model = progressBar.getModel();
	b = getBorderInsets(c);

	x = b.left;
	y = b.top;
	innerSize = new Dimension(size.width - (b.left + b.right),
				  size.height - (b.top + b.bottom));

	g.setColor(c.getForeground());
	span = model.getMaximum() - model.getMinimum();
	length = 0;
	if (progressBar.getOrientation() == JProgressBar.HORIZONTAL) {
	    if (span != 0) {
	        bigWidth = innerSize.width;
	        bigValue = model.getValue();
	        fractionComplete = bigValue / span;
	        length = (int)(bigWidth * fractionComplete);
	    }
	    max = (x + length);
	    // The Main Fill 
	    g.fillRect(x, y, max, innerSize.height);
	    // Highlighting
	    //     over the unfilled portion
	    //     actually, draw all the way across - let others draw over it
	    g.setColor(MetalLookAndFeel.getControlShadow());
	    g.drawLine(x,y, innerSize.width,y);
	    //     line on left
	    if (model.getValue() == 0) { // probably haven't started yet
		g.setColor(MetalLookAndFeel.getControlShadow());
	    } else { // Some portion of bar is filled
		g.setColor(MetalLookAndFeel.getPrimaryControlDarkShadow());
	    }
	    g.drawLine(x,y, x,y+innerSize.height);
	    //     over the filled portion
	    //     get color from leftline above
	    g.fillRect(x,y, max,y);
	} else {
	    if (span != 0) {
	        bigHeight = innerSize.height;
	        bigValue = model.getValue();
	        fractionComplete = bigValue / span;
	        length = (int)(bigHeight * fractionComplete);
	    }
	    min = (innerSize.height + y) - length;

	    // The Main Fill
	    g.fillRect(x, min, innerSize.width, length);
	    // Highlighting.
	    //     left of the unfilled portion
	    //     actually, draw all the way down - let others draw over it
	    g.setColor(MetalLookAndFeel.getControlShadow());
	    g.drawLine(x,y, x,y+innerSize.height);
	    //     line on bottom
	    if (model.getValue() == 0) { // probably haven't started yet
		g.setColor(MetalLookAndFeel.getControlShadow());
	    } else { // Some portion of bar is filled
		g.setColor(MetalLookAndFeel.getPrimaryControlDarkShadow());
	    }
	    g.drawLine(x,                y+innerSize.height-1,
		       x+innerSize.width,y+innerSize.height-1);
	    //     left of the filled portion
	    //     pick up color from the bottom line
	    g.fillRect(x, min, 1, length);
	}

	// What a pile of *%#$! this is.
	// It should use a proper border and the defaults mechanism
	if (progressBar.isBorderPainted()) {
	    g.setColor(MetalLookAndFeel.getControlDarkShadow());
	    g.drawRect(0,0, size.width-1, size.height-1);
	}
    }
 
    // Tightly tied to the "if (progressBar.isBorderPainted())" code in
    //  the paint() method. This is dumb. See comments about paint().
    public Insets getBorderInsets(JComponent c) {
	JProgressBar bar = (JProgressBar)c;
	if (bar.isBorderPainted()) {
	    return new Insets(1, 1, 1, 1);
	} else {
	    return new Insets(0, 0, 0, 0);
	}
    }


}
