/*
 * @(#)MetalToggleButtonUI.java	1.6 98/02/02
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
 
package java.awt.swing.plaf.metal;

import java.awt.*;
import java.awt.event.*;
import java.awt.swing.plaf.basic.BasicToggleButtonUI;

import java.awt.swing.*;
import java.awt.swing.border.*;
import java.awt.swing.plaf.*;
import java.awt.swing.*;

import java.io.Serializable;

/**
 * MetalToggleButton implementation
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.6 02/02/98
 * @author Tom Santos
 */
public class MetalToggleButtonUI extends BasicToggleButtonUI {
    private static Color selectedColor;
    private static Color disabledTextColor;
    private static Color focusColor;

    protected final static Insets defaultMargin = new Insets(2,14,2,14);

    private static final MetalToggleButtonUI metalToggleButtonUI = new MetalToggleButtonUI();

    public static ComponentUI createUI(JComponent b) {
        return metalToggleButtonUI;
    }

    public void installUI(JComponent c) {
        super.installUI(c);
	selectedColor = UIManager.getColor("ToggleButton.selected");
	disabledTextColor = UIManager.getColor("ToggleButton.disabledText");
	focusColor = UIManager.getColor("ToggleButton.focus");
    }

    protected void paintButtonPressed(Graphics g, AbstractButton b) {
        Dimension size = b.getSize();
	g.setColor( selectedColor );
	g.fillRect( 0, 0, size.width, size.height );
    }

    public Insets getDefaultMargin( AbstractButton b ) {
        return defaultMargin;
    }

    static class MetalToggleButtonBorder extends MetalButtonBorder {
        public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
	    JToggleButton button = (JToggleButton)c;
	    ButtonModel model = button.getModel();

	    if ( model.isPressed() && model.isArmed() ) {
	        MetalUtils.drawPressed3DBorder( g, x, y, w, h );
	    } else if (button.hasFocus()) {
	        MetalUtils.drawActiveButtonBorder( g, x, y, w, h );
	    } else if ( model.isSelected() ) {
	        MetalUtils.drawDark3DBorder( g, x, y, w, h );
	    } else {
	        MetalUtils.drawFlush3DBorder( g, x, y, w, h );
	    }
	}
    }

    protected void paintFocus(Graphics g, Dimension size){
        g.setColor(toggleButtonFocus);
	g.drawRect(2,2, size.width-5, size.height-5);	
    }
}
