/*
 * @(#)MetalToolBarBorder.java	1.1 98/02/05
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.metal;

import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Component;
import java.awt.swing.*;
import java.awt.swing.border.AbstractBorder;

/**
 * Draws the Metal ToolBar border.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.1 02/05/98
 * @author Jeff Shapiro
 */
public class MetalToolBarBorder extends AbstractBorder
{
    protected static MetalBumps bumps = new MetalBumps( 10, 10,
					     MetalLookAndFeel.getControlHighlight(),
					     MetalLookAndFeel.getControlDarkShadow(),
					     MetalLookAndFeel.getMenuBackground() );

    public void paintBorder( Component c, Graphics g, int x, int y, int w, int h )
    {
	g.translate( x, y );

	if ( ((JToolBar) c).isFloatable() )
	{
	    if ( c.getSize().height <= c.getSize().width )  // horizontal
	    {
		bumps.setBumpArea( 10, c.getSize().height - 6 );
		bumps.paintIcon( c, g, 2, 4 );
	    }
	    else  // vertical
	    {
		bumps.setBumpArea( c.getSize().width - 6, 10 );
		bumps.paintIcon( c, g, 4, 2 );
	    }

	}


	g.translate( -x, -y );
    }

    public Insets getBorderInsets( Component c )
    {
        Insets borderInsets = new Insets( 2, 2, 2, 2 );

	if ( ((JToolBar) c).isFloatable() )
	{
	    if ( c.getSize().height <= c.getSize().width )  // horizontal
	    {
	        borderInsets.left = 16;
	    }
	    else  // vertical
	    {
	        borderInsets.top = 16;
	    }
	}

        return borderInsets;
    }
}
