/*
 * @(#)MotifScrollBarButton.java	1.9 98/02/02
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.motif;

import java.awt.swing.*;
import java.awt.swing.event.*;
import java.awt.swing.plaf.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Motif scroll bar button.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 */
public class MotifScrollBarButton extends JButton implements SwingConstants
{
    private int direction = NORTH;
    private Color darkShadow = UIManager.getColor("controlShadow");
    private Color lightShadow = UIManager.getColor("controlLtHighlight");

    public MotifScrollBarButton(int direction)
    {
	switch (direction) {
	case NORTH:
	case SOUTH:
	case EAST:
	case WEST:
	    this.direction = direction;
	    break;
	default:
	    throw new IllegalArgumentException("invalid direction");
	}

	setRequestFocusEnabled(false);
	setOpaque(true);
	setBackground(UIManager.getColor("ScrollBar.background"));
	setForeground(UIManager.getColor("ScrollBar.foreground"));
    }


    public Dimension getPreferredSize() {
	switch (direction) {
	case NORTH: 
	case SOUTH: 
	    return new Dimension(11, 12);
	case EAST:
	case WEST:
	default:
	    return new Dimension(12, 11);
	}
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    public Dimension getMaximumSize() {
	return getPreferredSize();
    }

    public boolean isFocusTraversable() {
	return false;
    }


    public void paint(Graphics g) 
    {
	int w = getWidth();
	int h = getHeight();

	if (isOpaque()) {
	    g.setColor(getBackground());
	    g.fillRect(0, 0, w, h);
	}

	boolean isPressed = getModel().isPressed();
	Color lead = (isPressed) ? darkShadow : lightShadow;
	Color trail = (isPressed) ? lightShadow : darkShadow;
	Color fill = getForeground();

	int cx = w / 2;
	int cy = h / 2;
	int s = Math.min(w, h);

	switch (direction) {
	case NORTH:
	    g.setColor(lead);
	    g.drawLine(cx, 0, cx, 0);
	    for (int x = cx - 1, y = 1, dx = 1; y <= s - 2; y += 2) {
		g.setColor(lead);
		g.drawLine(x, y, x, y);
		if (y >= (s - 2)) {
		    g.drawLine(x, y + 1, x, y + 1);
		}
		g.setColor(fill);
		g.drawLine(x + 1, y, x + dx, y);
		if (y < (s - 2)) {
		    g.drawLine(x, y + 1, x + dx + 1, y + 1);
		}
		g.setColor(trail);
		g.drawLine(x + dx + 1, y, x + dx + 1, y);
		if (y >= (s - 2)) {
		    g.drawLine(x + 1, y + 1, x + dx + 1, y + 1);
		}
		dx += 2;
		x -= 1;
	    }
	    break;

	case SOUTH:
	    g.setColor(trail);
	    g.drawLine(cx, s, cx, s);
	    for (int x = cx - 1, y = s - 1, dx = 1; y >= 1; y -= 2) {
		g.setColor(lead);
		g.drawLine(x, y, x, y);
		if (y <= 2) {
		    g.drawLine(x, y - 1, x + dx + 1, y - 1);
		}
		g.setColor(fill);
		g.drawLine(x + 1, y, x + dx, y);
		if (y > 2) {
		    g.drawLine(x, y - 1, x + dx + 1, y - 1);
		}
		g.setColor(trail);
		g.drawLine(x + dx + 1, y, x + dx + 1, y);
		
		dx += 2;
		x -= 1;
	    }
	    break;

	case EAST:
	    g.setColor(lead);
	    g.drawLine(s, cy, s, cy);
	    for (int y = cy - 1, x = s - 1, dy = 1; x >= 1; x -= 2) {
		g.setColor(lead);
		g.drawLine(x, y, x, y);
		if (x <= 2) {
		    g.drawLine(x - 1, y, x - 1, y + dy + 1);
		}
		g.setColor(fill);
		g.drawLine(x, y + 1, x, y + dy);
		if (x > 2) {
		    g.drawLine(x - 1, y, x - 1, y + dy + 1);
		}
		g.setColor(trail);
		g.drawLine(x, y + dy + 1, x, y + dy + 1);

		dy += 2;
		y -= 1;
	    }
	    break;

	case WEST:
	    g.setColor(trail);
	    g.drawLine(0, cy, 0, cy);
	    for (int y = cy - 1, x = 1, dy = 1; x <= s - 2; x += 2) {
		g.setColor(lead);
		g.drawLine(x, y, x, y);
		if (x >= (s - 2)) {
		    g.drawLine(x + 1, y, x + 1, y);
		}
		g.setColor(fill);
		g.drawLine(x, y + 1, x, y + dy);
		if (x < (s - 2)) {
		    g.drawLine(x + 1, y, x + 1, y + dy + 1);
		}
		g.setColor(trail);
		g.drawLine(x, y + dy + 1, x, y + dy + 1);
		if (x >= (s - 2)) {
		    g.drawLine(x + 1, y + 1, x + 1, y + dy + 1);
		}
		dy += 2;
		y -= 1;
	    }
	    break;
	}
    }
}


