/*
 * @(#)HTMLFactory.java	1.8 98/01/19
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package java.awt.swing.text.html;

import java.awt.swing.text.*;

/**
 * A factory to build view fragments for html.
 *
 * @author  Timothy Prinzing
 * @version 1.8 01/19/98
 */
class HTMLFactory implements ViewFactory, HTMLDefs {
    
    /**
     * Creates a view from an element.
     *
     * @param elem the element
     * @return the view
     */
    public View create(Element elem) {
	String kind = elem.getName();
	if (kind != null) {
	    String ikind = kind.intern();
	    if (ikind == AbstractDocument.ContentElementName) {
		// text content
		return new LabelView(elem);
	    } else if ((ikind == AbstractDocument.ParagraphElementName) || (ikind == DT)) {
		// paragraph
		return new ParagraphView(elem);
	    } else if ((ikind == MENU) || (ikind == DIR) ||
		       (ikind == UL)   || (ikind == OL)) {
		return new ListView(elem);
	    } else if (ikind == AbstractDocument.SectionElementName) {
		return new HTMLRootView(elem, View.Y_AXIS);
	    } else if (ikind == PRELINE) {
		return new LineView(elem);
	    } else if ((ikind == LI) || (ikind == DL)  ||
		       (ikind == DD) || (ikind == PRE)) {
		// vertical box
		return new HTMLBoxView(elem, View.Y_AXIS);
	    } else if ((ikind == FORM)) {
		// horizontal box
		return new HTMLBoxView(elem, View.X_AXIS);
	    } else if (ikind==StyleConstants.ComponentElementName) {
		return new ComponentView(elem);
	    } else if (ikind==IMG) {
		return new ImageView(elem);
	    } else if (ikind == HR) {
		return new HRuleView(elem);
	    } else if (ikind == TABLE ) {
	    	return new TableView(elem);
	    }
	}

	// don't know how to build this....
	System.err.println("don't know how to build a: " + kind);
	return null;
    }

}

