/*
 * @(#)RuntimePermission.java	1.19 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
 
package java.lang;

import java.security.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * This class is for runtime permissions. A RuntimePermission 
 * contains a name but
 * no actions list; you either have the named permission 
 * or you don't.
 * 
 * <P>
 * The name is the name of the runtime permission ("exit",
 * "setFactory",
 * "print.queueJob", etc). The naming
 * convention follows the  hierarchical property naming convention.
 * Also, an asterisk 
 * may appear at the end of the name, following a ".", or by itself, to 
 * signify a wildcard match. For example: "package.*" or "*" is valid, 
 * "*package" or "a*b" is not valid.
 * <P>
 * @see java.security.BasicPermission
 * @see java.security.Permission
 * @see java.security.Permissions
 * @see java.security.PermissionCollection
 * @see java.lang.SecurityManager
 *
 * @version 1.19 98/03/18
 *
 * @author Marianne Mueller
 * @author Roland Schemers
 */

public final class RuntimePermission extends BasicPermission {

    /** use serialVersionUID from JDK 1.2 for interoperability */
    private static final long serialVersionUID = 7399184964622342223L;

    /**
     * Creates a new RuntimePermission with the specified name.
     * The name is the symbolic name of the RuntimePermission, such as
     * "exit", "setFactory", etc. An asterisk 
     * may appear at the end of the name, following a ".", or by itself, to 
     * signify a wildcard match.
     *
     * @param name the name of the RuntimePermission.
     */

    public RuntimePermission(String name)
    {
	super(name);
    }

    /**
     * Creates a new RuntimePermission object with the specified name.
     * The name is the symbolic name of the RuntimePermission, and the
     * actions String is currently unused and should be null. This
     * constructor exists for use by the <code>Policy</code> object
     * to instantiate new Permission objects.
     *
     * @param name the name of the RuntimePermission.
     * @param actions should be null.
     */

    public RuntimePermission(String name, String actions) 
    {
	super(name, actions);
    }
}
