/*
 * @(#)ArrayLocator.java	1.3 98/03/18
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.sql;
 
/**
 * JDBC 2.0
 *
 * <p>An ArrayLocator is a transaction duration reference to an array in the 
 * DBMS server.
 */

public interface ArrayLocator {

  /**
  * Return the SQL type of elements of the array designated by 
* this locator.
  *
  * @returns object representing SQL type of array element
  */
  SQLType getBaseType();

  /**
  * Retrieve from the database the SQL Array 
* designated by the locator.  
  * Use the type-map associated with the connection 
* for customizations of the type-mappings.
  *
  * @return an array containing the ordered elements of 
* the SQL Array designated by the locator.
  */
  Object[] getArray();

  /**
  * Retrieve from the database the SQL Array designated by 
  * the locator. Use the given @map for type-map customizations.
  *
  * @param map contains mapping of SQL user-defined types to 
* classes
  * @return an array containing the ordered elements of 
* the SQL Array 
  * designated by the locator.
  */
  Object[] getArray(java.util.Map map);

  /**
  * Materialize the item designated by the locator as a ResultSet
  * that returns a "row" for each element of the array, in which
  * the single "column" of that "row" contains that array element.
  * The rows are ordered in ascending order of the array-element
  * indexes.
  *  
  * @return a result set containing the elements of the array
  */
  ResultSet getResultSet ();

  /**
  * Materialize the item designated by the locator as a ResultSet
  * that returns a "row" for each element of the array, in which
  * the single "column" of that "row" contains that array element.
  * The rows are ordered in ascending order of the array-element
  * indexes.
  * Use the given @map for type-map customizations.
  *
  * @param map contains mapping of SQL user-defined types to 
* classes
  *  
  * @return a result set containing the elements of the array
  */
  ResultSet getResultSet (java.util.Map map);
}
