/*
 * @(#)BlobLocator.java	1.3 98/03/18
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.sql;

import java.io.InputStream;
 
/**
 * JDBC 2.0
 *
 * <p>A BlobLocator is a transaction duration reference to a binary large 
 * object in the DBMS server.
 */

public interface BlobLocator {

  /**
  * The length of the Binary Large Object in bytes.
  *
  * @return length of the BLOB in bytes
  */
  long length();

  /**
  * Return copy of the substring of the BLOB at the requested 
* position.
  *
  * @param pos is the first byte of the substring to be extracted.
  * @param length is the number of consecutive bytes to be copied.
  *
  * @return a byte array containing a substring of the BLOB
  */
  byte[] getBytes(int pos, int length); 

  /**
  * Retrieve the entire BLOB as a stream.
  *
  * @return a stream containing the BLOB data
  */
  InputStream getBinaryStream ();
}


