/*
 * @(#)Any.java	1.13 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.Streamable;


/**
 * Class <code>Any</code> is a container for any data that can be
 * described in IDL or any IDL primitive type. <p>
 * 
 * <a name="anyOps">
 * For each primitive type X, the following operations exist: <p>
 *      <dl>
 *	<dt> void insert_X(X x);
 *	<dd> This operations allows the insertion of a
 *	  an instance <code>x</code> of primitive type <code>X</code>
 *        into the <code>Any</code>. 
 *	<dt> X extract_X();
 *	<dd> This operation allows the extraction of an instance of
 *	  type <code>X</code> from the <code>Any</code>. If the type
 *	  of the element contained in the <code>Any</code> is not
 *	  <code>X</code>, the operation throws a
 *	  <code>BAD_OPERATION</code> exception.
 *      </dl>
 *
 * @version 1.13, 03/18/98
 * @since   JDK1.2
 */

abstract public class Any {

  /**
   * returns the type of the element contained in the Any.
   *
   * @result		the TypeCode for the element in the Any
   */
   
  abstract public TypeCode type();

  /**
   * sets the type of the element to be contained in the Any.
   *
   * @param t		the TypeCode for the element in the Any
   */
   
  abstract public void type(TypeCode t);

  /**
   * checks for equality between Anys.
   *
   * @param a	the Any to be compared with.
   * @result		true if the Anys are equal, false otherwise.
   */
   
  abstract public boolean equal(Any a);

  ///////////////////////////////////////////////////////////////////////////
  // marshaling/unmarshaling routines

  /**
   * reads off the value of an any using a typecode
   *
   * @param is		the Input Stream to read value of Any from
   * @param t		the typecode of the value to be read
   */

  abstract public void   read_value(InputStream is, TypeCode t)
    throws MARSHAL;

  /**
   * writes out the typecode and value of an Any
   *
   * @param os		the OutputStream to marshal value of Any into
   */

  abstract public void   write_value(OutputStream os) throws MARSHAL;

  /**
   * returns an output stream that an Any value can be marshaled into. 
   *
   * @result		the OutputStream to marshal value of Any into
   */

  abstract public OutputStream  create_output_stream();

  /**
   * returns an input stream that an Any value can be marshaled out of.
   *
   * @result		the InputStream to marshal value of Any out of.
   */

  abstract public InputStream  create_input_stream();

  ///////////////////////////////////////////////////////////////////////////
  // insertion of streamables 

  /**
   * takes a streamable and inserts it into the any
   *
   * @param s		the streamable to insert
   */

  abstract public void insert_Streamable(Streamable s);


  ///////////////////////////////////////////////////////////////////////////
  // basic insertion/extraction operations

  abstract public short    extract_short() throws BAD_OPERATION;
  abstract public void     insert_short(short s);

  abstract public int      extract_long() throws BAD_OPERATION;
  abstract public void     insert_long(int l);

  abstract public long     extract_longlong() throws BAD_OPERATION;
  abstract public void     insert_longlong(long l) throws BAD_OPERATION;

  abstract public short    extract_ushort() throws BAD_OPERATION;
  abstract public void     insert_ushort(short s);

  abstract public int      extract_ulong() throws BAD_OPERATION;
  abstract public void     insert_ulong(int l);

  abstract public long     extract_ulonglong() throws BAD_OPERATION;
  abstract public void     insert_ulonglong(long l);

  abstract public float    extract_float() throws BAD_OPERATION;
  abstract public void     insert_float(float f);

  abstract public double   extract_double() throws BAD_OPERATION;
  abstract public void     insert_double(double d);

  abstract public boolean  extract_boolean() throws BAD_OPERATION;
  abstract public void     insert_boolean(boolean b);

  abstract public char     extract_char() throws BAD_OPERATION;
  abstract public void     insert_char(char c);

  abstract public byte     extract_octet() throws BAD_OPERATION;
  abstract public void     insert_octet(byte b);

  abstract public char     extract_wchar() throws BAD_OPERATION;
  abstract public void     insert_wchar(char c);

  abstract public Any      extract_any() throws BAD_OPERATION;
  abstract public void     insert_any(Any a);

  abstract public String   extract_string() throws BAD_OPERATION;
  abstract public void     insert_string(String s);
  
  abstract public String   extract_wstring() throws BAD_OPERATION;
  abstract public void     insert_wstring(String s);
  
  abstract public org.omg.CORBA.Object extract_Object() throws BAD_OPERATION;
  abstract public void                 insert_Object(org.omg.CORBA.Object o);
  abstract public void                 insert_Object(org.omg.CORBA.Object o,
						     TypeCode t);

  abstract public TypeCode extract_TypeCode() throws BAD_OPERATION;
  abstract public void	   insert_TypeCode(TypeCode t);

  abstract public Principal extract_Principal() throws BAD_OPERATION;
  abstract public void	   insert_Principal(Principal p);

}
