/*
 * @(#)Context.java	1.12 98/03/18
 *
 * Copyright 1996, 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/**  
 * The Context class is used in the request operations to describe
 * specify the context object in which context strings must be
 * resolved before being sent along with the request invocation.
 * 
 * @version 1.12, 03/18/98
 * @since   JDK1.2
 */

public abstract class Context {

  /**
   * name accessor.
   *
   * @return			name of the context.
   */

  public abstract String context_name();

  
  /**
   * parent accessor.
   *
   * @return			parent of this context.
   */

  public abstract Context parent();

  /**
   * create_child creates a context with the current context set as
   * it's parent.
   *
   * @param child_ctx_name	string to be set as name of the context.
   * @return 			the context object created.
   */

  public abstract Context create_child(String child_ctx_name);

  /**
   * set_one_value sets the value of the string property name to the
   * Any provided.  
   *
   * @param propname		name of the property to be set.
   * @param propvalue		Any to set as value of the property.
   */

  public abstract void set_one_value(String propname, Any propvalue);

  /**
   * The set_values operation sets one or more property values in the
   * context object. In the NVList, the flags field must be set to
   * zero, and the TypeCode field associated with an attribute value
   * must be TC_string.
   *
   * @param values		the NVList describing the property
   * 				names and values to be set. 
   */

  public abstract void set_values(NVList values);

  /**
   * The delete_values operation deletes the specified property
   * value(s) values from the context object. If propName has a
   * trailing wildcard character ("*"), then all property names that
   * match will be deleted. Search scope is always limited to the
   * specified context object. If no matching property is found, an
   * exception is returned.
   *
   * @param propname		name of the property to be deleted.
   */

  public abstract void delete_values(String propname);

  /**
   *
   * The get_values operation retrieves the specified context property
   * value(s). It allows for wildcard searches as well. If the
   * property is not found at the indicated level, the search
   * continues up the context object tree until a match is found or
   * all context objects in the chain have been exhausted. If no
   * properties are found an error is returned, and no property list
   * is returned.
   *
   * @param start_scope		indicates the context
   *				object level at which to initiate the
   *				search for the specified properties
   *				(e.g. "_USER", "_SYSTEM"). Valid scope
   *				names are implementation-specific. If
   *				scope name is omitted, the search
   *				begins with the specified context
   *				object. If the specified scope name is
   *				not found, an exception is returned. 
   * @param op_flags		if the CTX_RESTRICT_SCOPE flag is
   *				specified, searching is limited to the
   *				specified search scope or context
   *				object. 
   * @param pattern		the property name whose values are to
   *				be retrieved. If pattern has a
   *				trailing wildcard character ("*"),
   *				then all matching properties and their
   *				values are returned. 
   * @result			all the matching properties and values
   *				in the form of a NVList.
   *
   */

  abstract public NVList get_values(String start_scope, int op_flags, 
				String pattern);
};

