/*
 * @(#)Request.java	1.14 98/03/18
 *
 * Copyright 1996, 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/** 
 * The Request pseudo-object is the cornerstone of the ORB Dynamic
 * Invocation interface (DII). The DII allows dynamic creation and
 * invocation of requests to objects, which does not require a stub
 * for the server interface type to be available.
 * A client using the DII to
 * send a request to an object obtains the same semantics as a client
 * using the operation stub generated from the type specification. A
 * request consists of an object reference, an operation, and a list
 * of parameters. In the Dynamic Invocation
 * interface, parameters in a request are supplied as elements of a
 * list. Each element is an instance of a NamedValue. Each parameter
 * is passed in its native data form. 
 * Request instances are created by invoking the _request() or _create_request()
 * operations on the target CORBA object reference.
 *
 * @see org.omg.CORBA.NamedValue
 *
 * @version 1.14 03/18/98
 */

public abstract class Request {

  /**
   * The target accessor
   *
   * @return			the target object reference on which
   *				the operation is to be invoked.
   */

  public abstract org.omg.CORBA.Object target();

  /**
   * The operation name accessor
   *
   * @return			the name of the operation to be invoked
   */

  public abstract String operation();

  /**
   * The arguments accessor
   *
   * @return			the arguments of the operation, in the
   *				form of an NVList.
   *
   */

  public abstract NVList arguments();
  
  /**
   * The result accessor
   *
   * @return			the NamedValue containing the result 
   *				of the operation.
   */

  public abstract NamedValue result();
  
  /**
   * The environment accessor
   *
   * @return			the environment object that will contain
   *				information about any exceptions that might
   *				might be returned on invocation of the
   *				request
   */

  public abstract Environment env();
  
  /**
   * The exceptions accessor
   *
   * @return			the list of exceptions the operation
   *				might throw
   */

  public abstract ExceptionList exceptions();
  
  /**
   * The context list accessor
   *
   * @return			the list of context strings whose values
   *				need to be resolved and sent with the
   *				invocation. 
   */

  public abstract ContextList contexts();
  
  /**
   * The context object accessor
   *
   * @return			the context object that is to be used
   *				to resolve any context strings whose
   *				values need to be sent with the invocation.
   */

  public abstract Context ctx();
  
  /**
   * The context object modifier
   *
   * @param c		        the new context object to be used for 
   *				resolving context strings.
   */

  public abstract void ctx(Context c);
  

  /**
   * The add_in_arg operation adds an "in" argument.
   * 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_in_arg();

  /**
   * The add_named_in_arg operation adds an "in" argument with the given name.
   * 
   * @param name		the name of the argument being added. 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_named_in_arg(String name);

  /**
   * The add_inout_arg operation adds an "inout" argument.
   * 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_inout_arg();

  /**
   * The add_inout_arg operation adds an "inout" argument with the given name.
   * 
   * @param name		the name of the argument being added. 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_named_inout_arg(String name);


  /**
   * The add_out_arg operation adds an "out" argument.
   * 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_out_arg();

  /**
   * The add_named_out_arg operation adds an "out" argument with the given name.
   * 
   * @param name		the name of the argument being added. 
   * @return			an Any which contains the
   * 				argument added.
   */

  public abstract Any add_named_out_arg(String name);

  /**
   * The set_return_type operation sets the typecode for the result
   * value of the operation.
   * 
   * @param tc			typecode for the result value.
   */

  public abstract void set_return_type(TypeCode tc);

  /**
   * The return_value operation returns the Any that contains the
   * result of the operation. 
   * 
   * @result			Any containing the result value.
   */

  public abstract Any return_value();
  
  /**
   * The invoke operation does a synchronous invocation using the
   * information in the Request object. Exception information is
   * placed into the Request environment object. 
   */

  public abstract void invoke();

  /**
   * The send_oneway operation does a oneway invocation on the
   * request. In other words, it does not expect or wait for a
   * response. Note that this can be used even if the operation was
   * not declared as oneway in the IDL declaration. No response or
   * exception information is returned. 
   */

  public abstract void send_oneway();
  
  /**
   * The send_deferred operation does an asynchronous invocation on
   * the request. In other words, it does not wait for response, but
   * returns to the user. The user can then use the poll_response and
   * get_response methods to later get the result or exception
   * information for the invocation.
   */

  public abstract void send_deferred();
  
  /**
   * The poll_response operation supports the asynchronous invocation
   * mechanism (send_deferred). It allows the user to determine
   * whether a response has been received for the invocation triggered
   * earlier with the 'send_deferred' operation.
   *
   * @return			true if the operation response has
   * 				been received, false otherwise.
   */

  public abstract boolean poll_response();
  
  /**
   * The get_response operation supports the asynchronous invocation
   * mechanism (send_deferred). It allows the user to access the
   * response for the invocation triggered earlier with the
   * 'send_deferred' operation.
   * @exception WrongTransaction is thrown if get_response was invoked
   * from a different transaction's scope than the one from which the
   * request was originally sent. See the OMG Transaction Service specification
   * for details.
   */

  public abstract void get_response() throws WrongTransaction;

};
