/*
 * @(#)TRANSIENT.java	1.18 98/03/18
 *
 * Copyright 1995-1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/**
 * TRANSIENT is a CORBA standard exception. It is an unchecked exception,
 * i.e. it is a subclass of java.lang.RuntimeException.
 * It may be thrown by the ORB during the processing of any request.
 * It is usually thrown by the server to signal a transient failure
 * which might not occur again if the request is retried.
 * For more
 * information about the exact cause of the exception, the String
 * description, the minor code and the completion status of the
 * exception may be examined.
 * @version     1.18 03/18/98
 */

public final class TRANSIENT extends SystemException {
    /**
     * Constructs a TRANSIENT with a default minor code
     * of 0, a completion state of CompletionStatus.COMPLETED_NO,
     * and a null description.
     */  
    public TRANSIENT() {
        this("");
    }

    /**
     * Constructs a TRANSIENT with the specified description message,
     * a minor code of 0, and a completion state of COMPLETED_NO.
     * @param s the String containing a detail message
     */  
    public TRANSIENT(String s) {
        this(s, 0, CompletionStatus.COMPLETED_NO);
    }

    /**
     * Constructs a TRANSIENT with the specified
     * minor code and completion status.
     * @param minor the minor code
     * @param completed the completion status
     */  
    public TRANSIENT(int minor, CompletionStatus completed) {
        this("", minor, completed);
    }

    /**
     * Constructs a TRANSIENT with the specified description
     * message, minor code, and completion status.
     * @param s the String containing a description message
     * @param minor the minor code
     * @param completed the completion status
     */  
    public TRANSIENT(String s, int minor, CompletionStatus completed) {
        super(s, minor, completed);
    }
}
