/*
 * @(#)TypeCode.java	1.26 98/03/18
 *
 * Copyright 1996, 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

import org.omg.CORBA.TypeCodePackage.*;

/** 
 * A TypeCode represents the type information of a specific CORBA data
 * type. TypeCodes can be obtained from a CORBA Interface Repository,
 * or from the Holder classes generated for IDL types by the IDL compiler
 * They can also be dynamically created through methods on the ORB class.
 * TypeCodes are CORBA pseudo-objects but they can be passed as
 * general IDL parameters. <p>
 * 
 * TypeCodes are used in the dynamic invocation interface to indicate
 * the types of the actual arguments. They are used by an Interface
 * Repository to represent the type specifications that are part of
 * many OMG IDL declarations. Finally, they are crucial to the
 * semantics of the Any type. <p>
 *
 * Abstractly, TypeCodes consist of a "kind" field, and a set of
 * parameters appropriate for that kind. For example, the TypeCode
 * describing OMG IDL type long has kind TCKind.tk_long and no parameters.
 * The TypeCode describing OMG IDL type sequence<boolean,10> has kind
 * TCKind.tk_sequence and two parameters: 10 and boolean. <p>
 * 
 * The deprecated parameter() and param_count() methods are
 * not mapped.  <p>
 *
 * We extend the CORBA specification to allow all operations permitted
 * on a struct TypeCode on an exception TypeCode as well. <p>
 *
 * @version 1.26 03/18/98 
 */


public abstract class TypeCode {

  /**
   * The equal operation can be invoked on any TypeCode. Equal
   * TypeCodes are interchangeable, and give identical results when
   * TypeCode operations are applied to them.
   * 
   * @param tc 		the TypeCode to compare against.
   * @return 		true if the typecodes are indeed equal, false
   * 			otherwise.  
   */
  
  public abstract boolean equal(TypeCode tc);
  
  /** 
   * The kind operation can be invoked on any TypeCode. 
   *
   * @return 		a TCKind indicating the kind of the
   * 			TypeCode. This also determines what other
   * 			operations can be invoked on the TypeCode. 
   */
  
  public abstract TCKind kind();
  
  /**
   * The id operation can be invoked on object reference, structure,
   * union, enumeration, alias, and exception TypeCodes. It returns
   * the RepositoryId globally identifying the type. Object
   * reference and exception TypeCodes always have a RepositoryId.
   * Structure, union, enumeration, and alias TypeCodes obtained
   * from the Interface Repository or the ORB::create_operation_list
   * operation also always have a RepositoryId. Otherwise, the id
   * operation can return an empty string. 
   *
   * @return		the RepositoryId for the type. 
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract String id() throws BadKind;
  
  /**
   * The name operation can also be invoked on object reference,
   * structure, union, enumeration, alias, and exception TypeCodes.
   * It returns the simple name identifying the type within its
   * enclosing scope. Since names are local to a Repository, the
   * name returned from a TypeCode may not match the name of the
   * type in any particular Repository, and may even be an empty
   * string. 
   * 
   * @return		the name identifying the type.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract String name() throws BadKind;
  
  /**
   * The member_count operation can be invoked on structure, union,
   * and enumeration TypeCodes. 
   * <br>
   * We extend the CORBA specification to allow this operation on
   * exceptions as well.  
   * 			
   * @return		the number of members constituting the type. 
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract int member_count() throws BadKind;
  
  /**
   * The member_name operation can be invoked on structure, union,
   * and enumeration TypeCodes. It returns the simple name
   * of the member identified by index. Since names are local to a
   * Repository, the name returned from a TypeCode may not match the
   * name of the member in any particular Repository, and may even
   * be an empty string.
   * <br>
   * We extend the CORBA specification to allow this operation on
   * exceptions as well.  
   *
   * @param index	index of member for which name is required.
   * @return		simple name of the member identified by the
   *			index. 
   * @exception org.omg.CORBA.TypeCodePackage.Bounds raised if the index is equal or greater than
   *			the number of members constituting the type.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   * 			inappropriate TypeCode kind. 
   */
  
  public abstract String member_name(int index)
    throws BadKind, org.omg.CORBA.TypeCodePackage.Bounds;
  
  /**
   * The member_type operation can be invoked on structure and union
   * TypeCodes. It returns the TypeCode describing the type of the
   * member identified by index. 
   * <br>
   * We extend the CORBA specification to allow this operation on
   * exceptions as well.  
   *
   * @param index	index of member for which type is required.
   * @return		TypeCode of the member identified by the
   *			index. 
   * @exception org.omg.CORBA.TypeCodePackage.Bounds raised if the index is equal or greater than
   *			the number of members constituting the type.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract TypeCode member_type(int index)
    throws BadKind, org.omg.CORBA.TypeCodePackage.Bounds;
  
  /**
   * The member_label operation can only be invoked on union
   * TypeCodes. Member_label returns the label of the union member
   * identified by index. For the default member, the label is the
   * zero octet.
   *
   * @param index	index of member for which label is required.
   * @return		an Any describing the label of the member
   *			identified by the index. 
   * @exception org.omg.CORBA.TypeCodePackage.Bounds raised if the index is equal or greater than
   *			the number of members constituting the type.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on
   *			non-union typecode.
   */
  
  public abstract Any member_label(int index)
    throws BadKind, org.omg.CORBA.TypeCodePackage.Bounds;
  
  /**
   * The discriminator_type operation can only be invoked on union
   * TypeCodes. The discriminator_type operation returns the type of
   * all non-default member labels. 
   *
   * @return		the type of the non-default member labels.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on
   *			non-union typecode.
   */
  
  public abstract TypeCode discriminator_type()
    throws BadKind;
  
  /**
   * The default_index operation can only be invoked on union
   * TypeCodes. The default_index operation returns the index of the
   * default member, or -1 if there is no default member.
   *
   * @return		the index of the default member, or -1 if
   *			there is no default.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on
   *			non-union typecode.
   */
  
  public abstract int default_index() throws BadKind;
  
  /** 
   * The length operation can be invoked on string, sequence, and
   * array TypeCodes. For strings and sequences, it returns the
   * bound, with zero indicating an unbounded string or sequence.
   * For arrays, it returns number of elements in the array.
   *
   * @return		the bound for strings and sequences, and the
   *			number of elements for arrays.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract int length() throws BadKind;
  
  /**
   * The content_type operation can be invoked on sequence, array,
   * and alias TypeCodes. For sequences and arrays, it returns the
   * element type. For aliases, it returns the original type. Note
   * that multidimensional arrays are represented by nesting
   * TypeCodes, one per dimension. 
   *
   * @return		the element type for sequences and arrays, and
   *			the original type for aliases.
   * @exception org.omg.CORBA.TypeCodePackage.BadKind raised if the operation is invoked on an
   *			inappropriate TypeCode kind. 
   */
  
  public abstract TypeCode content_type() throws BadKind;
  
}
